/*****************************************************************************/
/* A callback object for use in scheduling completion of asynchronous        */
/* requests.                                                                 */
/*                                                                           */
/* Copyright (c) 2005 Harry Butterworth IBM Corporation                      */
/*                                                                           */
/* This program is free software; you can redistribute it and/or modify it   */
/* under the terms of the GNU General Public License as published by the     */
/* Free Software Foundation; either version 2 of the License, or (at your    */
/* option) any later version.                                                */
/*                                                                           */
/* This program is distributed in the hope that it will be useful, but       */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General */
/* Public License for more details.                                          */
/*                                                                           */
/* You should have received a copy of the GNU General Public License along   */
/* with this program; if not, write to the Free Software Foundation, Inc.,   */
/* 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA                   */
/*                                                                           */
/*****************************************************************************/

#include "usbback_callback.h"

void usbback_callback_serialiser_function( void * context )
{
    struct usbback_callback_serialiser * serialiser =
      (struct usbback_callback_serialiser *)context;

    unsigned long flags;

    spin_lock_irqsave( &serialiser->lock, flags );

    while
    (
        ( !list_empty( &serialiser->list ) )
        &&
        ( !serialiser->running )
    )
    {
        struct usbback_callback * callback =
          usbback_callback_link_to( serialiser->list.next );

        list_del_init( usbback_callback_to_link( callback ) );

        serialiser->running = 1;

        spin_unlock_irqrestore( &serialiser->lock, flags );

        usbback_callback_complete_synchronously( callback );

        spin_lock_irqsave( &serialiser->lock, flags );

        serialiser->running = 0;
    }

    spin_unlock_irqrestore( &serialiser->lock, flags );
}
