/*****************************************************************************/
/* A device object representing the low-level connection to a single         */
/* front-end.                                                                */
/*                                                                           */
/* Copyright (c) 2005 Harry Butterworth IBM Corporation                      */
/*                                                                           */
/* This program is free software; you can redistribute it and/or modify it   */
/* under the terms of the GNU General Public License as published by the     */
/* Free Software Foundation; either version 2 of the License, or (at your    */
/* option) any later version.                                                */
/*                                                                           */
/* This program is distributed in the hope that it will be useful, but       */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General */
/* Public License for more details.                                          */
/*                                                                           */
/* You should have received a copy of the GNU General Public License along   */
/* with this program; if not, write to the Free Software Foundation, Inc.,   */
/* 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA                   */
/*                                                                           */
/*****************************************************************************/

#ifndef USBBACK_DEVICE_H
#define USBBACK_DEVICE_H

#include <linux/device.h>
#include <asm-xen/ctrl_if.h>
#include <asm-xen/xen-public/io/usbif.h>

struct usbback_device;

extern struct usbback_device * usbback_device_dev_to( struct device * dev );

/* Interface used by discovery code: */

extern struct usbback_device * usbback_device_find_or_allocate
  ( u32 handle, int shutting_down );

extern void usbback_device_put( struct usbback_device * device );

extern void usbback_device_handle_control_message
(
    struct usbback_device * device,
    ctrl_msg_t            * msg,
    unsigned long           id
);

/* usbback_device_shutdown_all_phase_1 blocks until all devices have stopped */
/* using any front-end resources.                                            */

extern void usbback_device_shutdown_all_phase_1( void );

/* usbback_device_shutdown_all_phase_2 is called after xend has guaranteed   */
/* not to send any more control messages to release the existing devices.    */

extern void usbback_device_shutdown_all_phase_2( void );

/* Interface used by device driver: */

struct usbback_device_request
{
    struct list_head link;
    usbif_request_t  usbif_request;
    usbif_response_t usbif_response;
};

extern void usbback_device_attach_backend
(
    struct usbback_device * device,
    void                ( * request_handler )
      ( struct usbback_device_request * request, void * context ),
    void                ( * claim_port      )
      ( u32 port, char * path, void * context ),
    void                ( * release_port    )( u32 port, void * context ),
    void                ( * flush_requests )( void * context ),
    void                  * context
);

extern void usbback_device_detach_backend( struct usbback_device * device );

extern void usbback_device_complete_request
(
    struct usbback_device_request * request,
    s16                             status,
    unsigned long                   length
);

extern domid_t usbback_device_query_remote_domain_id
  ( struct usbback_device * device );

#endif
