/*****************************************************************************/
/* Resource which processes a usbback_device_request by translating it into  */
/* an URB and submitting it to the USB stack.                                */
/*                                                                           */
/* Copyright (c) 2005 Harry Butterworth IBM Corporation                      */
/*                                                                           */
/* This program is free software; you can redistribute it and/or modify it   */
/* under the terms of the GNU General Public License as published by the     */
/* Free Software Foundation; either version 2 of the License, or (at your    */
/* option) any later version.                                                */
/*                                                                           */
/* This program is distributed in the hope that it will be useful, but       */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General */
/* Public License for more details.                                          */
/*                                                                           */
/* You should have received a copy of the GNU General Public License along   */
/* with this program; if not, write to the Free Software Foundation, Inc.,   */
/* 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA                   */
/*                                                                           */
/*****************************************************************************/

#ifndef USBBACK_DRIVER_PORT_RESOURCE_H
#define USBBACK_DRIVER_PORT_RESOURCE_H

#include "usbback_callback.h"
#include "usbback_device.h"
#include "usbback_mapper.h"
#include "usbback_work.h"

struct usbback_driver_port;
struct usbback_driver_port_resource;

void usbback_driver_port_resource_init
(
    struct usbback_driver_port_resource * resource,
    struct usbback_driver_port          * port
);

void usbback_driver_port_resource_start_io
(
    struct usbback_driver_port_resource * resource,
    struct usbback_device_request       * request
);

int usbback_driver_port_resource_try_unlink
  ( struct usbback_driver_port_resource * resource, int unlink_id );

void usbback_driver_port_resource_flush_request
  ( struct usbback_driver_port_resource * resource );

typedef enum
{
    usbback_driver_port_resource_state_i,
    usbback_driver_port_resource_state_i_st,
    usbback_driver_port_resource_state_i_st_tu,
    usbback_driver_port_resource_state_i_st_ms,
    usbback_driver_port_resource_state_i_st_ms_tu,
    usbback_driver_port_resource_state_i_st_ms_us,
    usbback_driver_port_resource_state_i_st_ms_uc,
    usbback_driver_port_resource_state_i_st_ms_tu_us,
    usbback_driver_port_resource_state_i_st_ms_tu_us_uc
}
usbback_driver_port_resource_state;

struct usbback_driver_port_resource
{
    struct usbback_driver_port         * port;
    struct list_head                     link;
    spinlock_t                           lock;
    usbback_driver_port_resource_state   state;
    s16                                  status;
    unsigned long                        actual_length;
    struct usbback_device_request      * device_request;
    struct usbback_device_request      * completing_device_request;
    struct usbback_mapper_request        mapper_request;
    struct usbback_mapper_element        mapper_element[ 2 ];
    int                                  buffer;
    struct urb                         * urb;
    struct usbback_callback              submit_urb_1_callback;
    struct usbback_work                  unlink_urb_1_work;
    struct usbback_work                  complete_1_work;
};

#endif
