/*****************************************************************************/
/* Enhanced work queue service                                               */
/* Copyright (c) 2005 Harry Butterworth IBM Corporation                      */
/*                                                                           */
/* This program is free software; you can redistribute it and/or modify it   */
/* under the terms of the GNU General Public License as published by the     */
/* Free Software Foundation; either version 2 of the License, or (at your    */
/* option) any later version.                                                */
/*                                                                           */
/* This program is distributed in the hope that it will be useful, but       */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General */
/* Public License for more details.                                          */
/*                                                                           */
/* You should have received a copy of the GNU General Public License along   */
/* with this program; if not, write to the Free Software Foundation, Inc.,   */
/* 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA                   */
/*                                                                           */
/*****************************************************************************/

#include "usbback_work.h"

DEFINE_SPINLOCK( usbback_work_list_lock );

LIST_HEAD( usbback_work_list );

static void usbback_work_function( void * ignored );

DECLARE_WORK( usbback_work_work, usbback_work_function, NULL );

DECLARE_WAIT_QUEUE_HEAD( usbback_work_waitqueue );

LIST_HEAD( usbback_work_condition );

void usbback_work_wake_up( void )
{
    unsigned long flags;

    spin_lock_irqsave( &usbback_work_list_lock, flags );

    while( !list_empty( &usbback_work_condition ) )
    {
        list_del_init( usbback_work_condition.next );
    }

    spin_unlock_irqrestore( &usbback_work_list_lock, flags );

    wake_up( &usbback_work_waitqueue );
}

int usbback_work_schedule( struct usbback_work * work )
{
    int scheduled = 0;

    unsigned long flags;

    spin_lock_irqsave( &usbback_work_list_lock, flags );

    if( list_empty( &work->link ) )
    {
        list_add_tail( &work->link, &usbback_work_list );

        scheduled = 1;
    }

    spin_unlock_irqrestore( &usbback_work_list_lock, flags );

    if( scheduled )
    {
        usbback_work_wake_up();

        schedule_work( &usbback_work_work );
    }

    return scheduled;
}

static void usbback_work_function( void * ignored )
{
    unsigned long flags;

    spin_lock_irqsave( &usbback_work_list_lock, flags );

    while( !list_empty( &usbback_work_list ) )
    {
        struct usbback_work * work = list_entry
        (
            usbback_work_list.next,
            struct usbback_work,
            link
        );

        list_del_init( &work->link );

        spin_unlock_irqrestore( &usbback_work_list_lock, flags );

        usbback_work_perform_synchronously( work );

        spin_lock_irqsave( &usbback_work_list_lock, flags );
    }

    spin_unlock_irqrestore( &usbback_work_list_lock, flags );
}
