/*****************************************************************************/
/* usbfront_device is a device which represents a connection to a back-end.  */
/* The intent was for it to have an interface like a hardware USB host       */
/* controller device.                                                        */
/*                                                                           */
/* Copyright (c) 2005 Harry Butterworth IBM Corporation                      */
/*                                                                           */
/* This program is free software; you can redistribute it and/or modify it   */
/* under the terms of the GNU General Public License as published by the     */
/* Free Software Foundation; either version 2 of the License, or (at your    */
/* option) any later version.                                                */
/*                                                                           */
/* This program is distributed in the hope that it will be useful, but       */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General */
/* Public License for more details.                                          */
/*                                                                           */
/* You should have received a copy of the GNU General Public License along   */
/* with this program; if not, write to the Free Software Foundation, Inc.,   */
/* 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA                   */
/*                                                                           */
/*****************************************************************************/

#ifndef USBFRONT_DEVICE_H
#define USBFRONT_DEVICE_H

#include <linux/device.h>
#include <linux/usb.h>
#include "../../usb/core/hcd.h"
#include <asm-xen/ctrl_if.h>
#include <asm-xen/xen-public/io/usbif.h>

struct usbfront_device;

struct usbfront_device * usbfront_device_dev_to( struct device * dev );

/* Interface used by discovery code: */

struct usbfront_device * usbfront_device_find_or_allocate
  ( u32 handle, int shutting_down );

void usbfront_device_put( struct usbfront_device * device );

void usbfront_device_handle_control_message
(
    struct usbfront_device * device,
    ctrl_msg_t             * msg,
    unsigned long            id
);

/* usbfront_device_shutdown_all_phase_1 blocks until all devices have        */
/* quiesced locally initiated control message traffic.                       */

void usbfront_device_shutdown_all_phase_1( void );

/* usbfront_device_shutdown_all_phase_2 is called once the the driver status */
/* down request has been acknowledged when there will be no more control     */
/* messages from the domain controller and the front-end resources have been */
/* released by the back ends.                                                */

void usbfront_device_shutdown_all_phase_2( void );

/* Interface used by driver code: */

int usbfront_device_query_port_count( struct usbfront_device * device );

int usbfront_device_query_port_status_changed
  ( struct usbfront_device * device, int port_index );

struct usb_port_status usbfront_device_query_port_status
  ( struct usbfront_device * device, int port_index );

/* Used to turn on power to a port. */
void usbfront_device_set_port_power
  ( struct usbfront_device * device, int port_index );

/* Used to reset a port. Completes asynchronously with notification via the  */
/* port status.                                                              */
void usbfront_device_set_port_reset
  ( struct usbfront_device * device, int port_index );

void usbfront_device_clear_port_enable
  ( struct usbfront_device * device, int port_index );

/* Used to ack a connection change */
void usbfront_device_clear_port_connection_change
  ( struct usbfront_device * device, int port_index );

/* Used to ack a reset change */
void usbfront_device_clear_port_reset_change
  ( struct usbfront_device * device, int port_index );

void usbfront_device_install_irq_handler
(
    struct usbfront_device * device,
    irqreturn_t ( * handler )
      ( int irq, void * context, struct pt_regs * regs ),
    void * context
);

/* Passed to usbfront_device_queue_request. Driver initializes usbif_request */
/* only.                                                                     */
struct usbfront_device_request
{
    struct list_head     link;
    usbif_request_t      usbif_request;
    usbif_response_t     usbif_response;
    void             ( * internal_completion )
    (
        struct usbfront_device         * device,
        struct usbfront_device_request * request
    );
};

void usbfront_device_queue_request
(
    struct usbfront_device         * device,
    struct usbfront_device_request * request
);

struct usbfront_device_request * usbfront_device_collect_completed_request
  ( struct usbfront_device * device );

void usbfront_device_remove_irq_handler( struct usbfront_device * device );

#endif
