/*****************************************************************************/
/* A device driver for usbfront_device devices.                              */
/*                                                                           */
/* Copyright (c) 2005 Harry Butterworth IBM Corporation                      */
/*                                                                           */
/* This program is free software; you can redistribute it and/or modify it   */
/* under the terms of the GNU General Public License as published by the     */
/* Free Software Foundation; either version 2 of the License, or (at your    */
/* option) any later version.                                                */
/*                                                                           */
/* This program is distributed in the hope that it will be useful, but       */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General */
/* Public License for more details.                                          */
/*                                                                           */
/* You should have received a copy of the GNU General Public License along   */
/* with this program; if not, write to the Free Software Foundation, Inc.,   */
/* 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA                   */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/* Based on                                                                  */
/*                                                                           */
/* arch/xen/drivers/usbif/frontend/main.c                                    */
/* pristine-linux-2.6.11/drivers/usb/host/uhci-hcd.c                         */
/* pristine-linux-2.6.11/drivers/usb/core/hcd-pci.c                          */
/*                                                                           */
/* original copyright notices follow...                                      */
/*****************************************************************************/

/*
 * Xen Virtual USB Frontend Driver 
 *
 * This file contains the first version of the Xen virtual USB hub
 * that I've managed not to delete by mistake (3rd time lucky!).
 *
 * Based on Linux's uhci.c, original copyright notices are displayed
 * below.  Portions also (c) 2004 Intel Research Cambridge
 * and (c) 2004, 2005 Mark Williamson
 *
 * Contact <mark.williamson@cl.cam.ac.uk> or
 * <xen-devel@lists.sourceforge.net> regarding this code.
 *
 * Still to be (maybe) implemented:
 * - migration / backend restart support?
 * - support for building / using as a module
 */

/*
 * Universal Host Controller Interface driver for USB.
 *
 * Maintainer: Johannes Erdfelt <johannes@erdfelt.com>
 *
 * (C) Copyright 1999 Linus Torvalds
 * (C) Copyright 1999-2002 Johannes Erdfelt, johannes@erdfelt.com
 * (C) Copyright 1999 Randy Dunlap
 * (C) Copyright 1999 Georg Acher, acher@in.tum.de
 * (C) Copyright 1999 Deti Fliegl, deti@fliegl.de
 * (C) Copyright 1999 Thomas Sailer, sailer@ife.ee.ethz.ch
 * (C) Copyright 1999 Roman Weissgaerber, weissg@vienna.at
 * (C) Copyright 2000 Yggdrasil Computing, Inc. (port of new PCI interface
 *               support from usb-ohci.c by Adam Richter, adam@yggdrasil.com).
 * (C) Copyright 1999 Gregory P. Smith (from usb-ohci.c)
 *
 * Intel documents this fairly well, and as far as I know there
 * are no royalties or anything like that, but even so there are
 * people who decided that they want to do the same thing in a
 * completely different way.
 *
 * WARNING! The USB documentation is downright evil. Most of it
 * is just crap, written by a committee. You're better off ignoring
 * most of it, the important stuff is:
 *  - the low-level protocol (fairly simple but lots of small details)
 *  - working around the horridness of the rest
 */

/*
 * Universal Host Controller Interface driver for USB.
 *
 * Maintainer: Alan Stern <stern@rowland.harvard.edu>
 *
 * (C) Copyright 1999 Linus Torvalds
 * (C) Copyright 1999-2002 Johannes Erdfelt, johannes@erdfelt.com
 * (C) Copyright 1999 Randy Dunlap
 * (C) Copyright 1999 Georg Acher, acher@in.tum.de
 * (C) Copyright 1999 Deti Fliegl, deti@fliegl.de
 * (C) Copyright 1999 Thomas Sailer, sailer@ife.ee.ethz.ch
 * (C) Copyright 1999 Roman Weissgaerber, weissg@vienna.at
 * (C) Copyright 2000 Yggdrasil Computing, Inc. (port of new PCI interface
 *               support from usb-ohci.c by Adam Richter, adam@yggdrasil.com).
 * (C) Copyright 1999 Gregory P. Smith (from usb-ohci.c)
 * (C) Copyright 2004 Alan Stern, stern@rowland.harvard.edu
 *
 * Intel documents this fairly well, and as far as I know there
 * are no royalties or anything like that, but even so there are
 * people who decided that they want to do the same thing in a
 * completely different way.
 *
 * WARNING! The USB documentation is downright evil. Most of it
 * is just crap, written by a committee. You're better off ignoring
 * most of it, the important stuff is:
 *  - the low-level protocol (fairly simple but lots of small details)
 *  - working around the horridness of the rest
 */

/*
 * (C) Copyright David Brownell 2000-2002
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <linux/device.h>
#include "usbfront_bus.h"
#include "usbfront_driver.h"
#include "usbfront_hcd_resource.h"
#include "usbfront_sll.h"
#include "usbfront_trace.h"

typedef enum
{
    usbfront_hcd_state_i
}
usbfront_hcd_state;

typedef enum
{
    usbfront_hcd_stimulus_uq /* urb enqueue */
}
usbfront_hcd_stimulus;

#define USBFRONT_HCD_RESOURCE_COUNT 16

struct usbfront_hcd
{
    spinlock_t                   lock;
    usbfront_hcd_state           state;
    struct usbfront_sll          urb_sll;
    struct list_head             free_resource_list;
    struct usbfront_hcd_resource resources[ USBFRONT_HCD_RESOURCE_COUNT ];
};

static void usbfront_hcd_handle_stimulus
(
    struct usbfront_hcd   * uhcd,
    usbfront_hcd_stimulus   stimulus,
    unsigned long         * flags
);

static struct usbfront_hcd * usbfront_hcd_hcd_to_uhcd( struct usb_hcd * hcd )
{
    return (struct usbfront_hcd *)( hcd->hcd_priv );
}

static struct usb_hcd * usbfront_hcd_to_hcd( struct usbfront_hcd * uhcd )
{
    return container_of( (void *)uhcd, struct usb_hcd, hcd_priv );
}

static struct usbfront_device *
  usbfront_hcd_hcd_to_usbfront_device( struct usb_hcd * hcd )
{
    return usbfront_device_dev_to( hcd_to_bus( hcd )->controller );
}

static int usbfront_hcd_start( struct usb_hcd * hcd )
{
    trace();

    int return_value = 0;

    struct usbfront_hcd * uhcd = usbfront_hcd_hcd_to_uhcd( hcd );

    memset( uhcd, 0, sizeof( *uhcd ) );

    spin_lock_init( &uhcd->lock );

    uhcd->state = usbfront_hcd_state_i;

    usbfront_sll_init( &uhcd->urb_sll );

    INIT_LIST_HEAD( &uhcd->free_resource_list );

    {
        int i;

        for( i = 0; i < USBFRONT_HCD_RESOURCE_COUNT; i++ )
        {
            struct usbfront_hcd_resource * resource = &uhcd->resources[ i ];

            return_value = usbfront_hcd_resource_init( resource, i );

            if( return_value != 0 )
            {
                goto EXIT_NO_RESOURCE;
            }

            list_add
            (
                usbfront_hcd_resource_to_link( resource ),
                &uhcd->free_resource_list
            );
        }
    }

    struct usb_device * root_hub_device = usb_alloc_dev( NULL, &hcd->self, 0 );

    if( root_hub_device == NULL )
    {
        return_value = -ENOMEM;

        goto EXIT_0;
    }

    /* The hc_driver layer shares this state variable between base and       */
    /* derived classes and we rely on side effects on it for correct         */
    /* operation. Joy.                                                       */

    hcd->state = USB_STATE_RUNNING;

    root_hub_device->speed = USB_SPEED_HIGH;

    if( ( return_value = hcd_register_root( root_hub_device, hcd ) ) != 0 )
    {
        goto EXIT_1;
    }

    return 0;

  EXIT_1:

    usb_put_dev( root_hub_device );

  EXIT_0:

  EXIT_NO_RESOURCE:

    while( !list_empty( &uhcd->free_resource_list ) )
    {
        struct usbfront_hcd_resource * resource = list_entry
        (
            uhcd->free_resource_list.next,
            struct usbfront_hcd_resource,
            link
        );

        list_del_init( usbfront_hcd_resource_to_link( resource ) );

        usbfront_hcd_resource_exit( resource );
    }

    return return_value;
}

static void usbfront_hcd_stop( struct usb_hcd * hcd )
{
    trace();

    struct usbfront_hcd * uhcd = usbfront_hcd_hcd_to_uhcd( hcd );

    while( !list_empty( &uhcd->free_resource_list ) )
    {
        struct usbfront_hcd_resource * resource = list_entry
        (
            uhcd->free_resource_list.next,
            struct usbfront_hcd_resource,
            link
        );

        list_del_init( usbfront_hcd_resource_to_link( resource ) );

        usbfront_hcd_resource_exit( resource );
    }
}

static int usbfront_hcd_get_frame_number( struct usb_hcd * hcd )
{
    trace();

    return 0;
}

static inline void usbfront_check_contiguous
  ( unsigned long buffer, unsigned long length )
{
    if( length != 0 )
    {
        unsigned long offset;

        for
        (
            offset = 0;
            offset < ( length + ( buffer & ~PAGE_MASK ) );
            offset += PAGE_SIZE
        )
        {
            unsigned long maddress1 =
              virt_to_machine( ( buffer & PAGE_MASK ) + offset );

            unsigned long maddress2 =
              virt_to_machine( buffer & PAGE_MASK ) + offset;

            ASSERT( maddress1 == maddress2 );
        }
    }
}

static int usbfront_hcd_urb_enqueue
(
    struct usb_hcd           * hcd,
    struct usb_host_endpoint * ep, /* FIXME: what's the intended use for ep? */
    struct urb               * urb,
    int                        mem_flags
)
{
    trace();

    usbfront_check_contiguous
      ( (unsigned long)urb->transfer_buffer, urb->transfer_buffer_length );

    struct usbfront_hcd * uhcd = usbfront_hcd_hcd_to_uhcd( hcd );

    unsigned long flags;

    spin_lock_irqsave( &uhcd->lock, flags );

    usbfront_slk_init( (struct usbfront_slk *)&urb->hcpriv );

    usbfront_sll_add_last
      ( &uhcd->urb_sll, (struct usbfront_slk *)&urb->hcpriv );

    usbfront_hcd_handle_stimulus( uhcd, usbfront_hcd_stimulus_uq, &flags );

    spin_unlock_irqrestore( &uhcd->lock, flags );

    return 0;
}

static int usbfront_hcd_urb_dequeue( struct usb_hcd * hcd, struct urb * urb )
{
    trace();

    struct usbfront_hcd * uhcd = usbfront_hcd_hcd_to_uhcd( hcd );

    int dequeued;

    {
        unsigned long flags;

        spin_lock_irqsave( &uhcd->lock, flags );

        dequeued = usbfront_sll_remove_slk
          ( &uhcd->urb_sll, (struct usbfront_slk *)&urb->hcpriv );

        spin_unlock_irqrestore( &uhcd->lock, flags );
    }

    if( dequeued )
    {
        urb->hcpriv = 0;

        urb->actual_length = 0;

        unsigned long flags;

        local_irq_save( flags );

        usb_hcd_giveback_urb( hcd, urb, 0 );

        local_irq_restore( flags );
    }
    else
    {
        usbfront_hcd_resource_dequeue_urb
          ( urb, usbfront_hcd_hcd_to_usbfront_device( hcd ) );
    }

    return 0;
}

static void usbfront_hcd_endpoint_disable
  ( struct usb_hcd * hcd, struct usb_host_endpoint * ep )
{
    trace();
}

static int usbfront_hcd_hub_status_data( struct usb_hcd * hcd, char * buf )
{
    int changed = 0;

    struct usbfront_device * device =
      usbfront_hcd_hcd_to_usbfront_device( hcd );

    int port_count = usbfront_device_query_port_count( device );

    int byte_count =
      ( 1 /* hub */ + port_count /* ports */ + 7 /* round up */ ) / 8;

    memset( buf, 0, byte_count );

    {
        int i;

        for( i = 0; i < port_count; i++ )
        {
            if( usbfront_device_query_port_status_changed( device, i ) )
            {
                buf[ ( i + 1 ) / 8 ] |= 1 << ( ( i + 1 ) % 8 );

                changed = 1;
            }
        }
    }

    return changed ? byte_count : 0;
}

static int usbfront_hcd_hub_control
(
    struct usb_hcd * hcd,
    u16              typeReq,
    u16              wValue,
    u16              wIndex,
    char           * buf,
    u16              wLength
)
{
    trace();

    struct usbfront_device * device =
      usbfront_hcd_hcd_to_usbfront_device( hcd );

    int return_value = 0;

    switch( typeReq )
    {
      case GetHubStatus:

        trace0( "GetHubStatus" );

        if( ( wValue != 0 ) || ( wIndex != 0 ) || ( wLength != 4 ) )
        {
            goto ERROR;
        }

        /* 2 bytes wHubStatus and 2 bytes wHubChange:        */
        /* Local power supply good, no overcurrect condition */
        /* No changes.                                       */

        memset( buf, 0, wLength );

        break;

      case GetPortStatus:

        trace0( "GetPortStatus" );

        if
        (
            ( wValue != 0 )
            ||
            ( wIndex > usbfront_device_query_port_count( device ) )
            ||
            ( wLength != 4 )
        )
        {
            goto ERROR;
        }

        {
            struct usb_port_status port_status =
              usbfront_device_query_port_status( device, wIndex - 1 );

            memcpy( buf, &port_status, wLength );
        }

        break;

      case GetHubDescriptor:

        trace0( "GetHubDescriptor" );

        {
            /* bDescLength             */
            /* bDescriptorType         */
            /* bNbrPorts               */
            /* wHubCharacteristics LSB */
            /* wHubCharacteristics MSB */
            /* bPwrOn2PwrGood          */
            /* bHubContrCurrent        */
            /* DeviceRemovable         */
            /* PortPwrCtrlMask         */

            /* See table 11.23.2.1 of the USB 2.0 specification. */

            char descriptor[] =
              { 0x09, 0x29, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0xFF };

            descriptor[ 2 ] = usbfront_device_query_port_count( device );

            memcpy
            (
                buf,
                &descriptor,
                min_t( size_t, sizeof( descriptor ), wLength )
            );
        }

        break;

      case SetPortFeature:

        trace0( "SetPortFeature" );

        if
        (
            (
                ( wIndex & 0x00FF )
                >
                usbfront_device_query_port_count( device )
            )
            ||
            ( wLength != 0 )
        )
        {
            goto ERROR;
        }

        switch( wValue )
        {
          case USB_PORT_FEAT_SUSPEND:
            trace0( "USB_PORT_FEAT_SUSPEND" );
            break;
          case USB_PORT_FEAT_RESET:
            trace0( "USB_PORT_FEAT_RESET" );
            usbfront_device_set_port_reset( device, wIndex - 1 );
            break;
          case USB_PORT_FEAT_POWER:
            trace0( "USB_PORT_FEAT_POWER" );
            usbfront_device_set_port_power( device, wIndex - 1 );
            break;
          default:
            trace1( "Unknown:%x", wValue );
            goto ERROR;
        }

        break;

      case ClearPortFeature:

        trace0( "ClearPortFeature" );

        if
        (
            (
                ( wIndex & 0x00FF )
                >
                usbfront_device_query_port_count( device )
            )
            ||
            ( wLength != 0 )
        )
        {
            goto ERROR;
        }

        switch( wValue )
        {
          case USB_PORT_FEAT_ENABLE:
            trace0( "USB_PORT_FEAT_ENABLE" );
            usbfront_device_clear_port_enable( device, wIndex - 1 );
            break;
          case USB_PORT_FEAT_SUSPEND:
            trace0( "USB_PORT_FEAT_SUSPEND" );
            break;
          case USB_PORT_FEAT_POWER:
            trace0( "USB_PORT_FEAT_POWER" );
            break;
          case USB_PORT_FEAT_INDICATOR:
            trace0( "USB_PORT_FEAT_INDICATOR" );
            break;
          case USB_PORT_FEAT_C_CONNECTION:
            trace0( "USB_PORT_C_CONNECTION" );
            usbfront_device_clear_port_connection_change( device, wIndex - 1 );
            break;
          case USB_PORT_FEAT_C_RESET:
            trace0( "USB_PORT_C_RESET" );
            usbfront_device_clear_port_reset_change( device, wIndex - 1 );
            break;
          case USB_PORT_FEAT_C_ENABLE:
            trace0( "USB_PORT_C_ENABLE" );
            break;
          case USB_PORT_FEAT_C_SUSPEND:
            trace0( "USB_PORT_C_SUSPEND" );
            break;
          case USB_PORT_FEAT_C_OVER_CURRENT:
            trace0( "USB_PORT_C_OVER_CURRENT" );
            break;
          default:
            trace1( "Unknown:%x", wValue );
            goto ERROR;
        }

        break;

      default:
        trace1( "Unknown:%x", typeReq );
      ERROR:
        return_value = -EPIPE;
    }

    return return_value;
}

static int usbfront_hcd_start_port_reset
  ( struct usb_hcd * hcd, unsigned port_num )
{
    trace();

    return 0;
}

static irqreturn_t usbfront_hcd_irq
  ( struct usb_hcd * hcd, struct pt_regs * regs );

static struct hc_driver usbfront_hc_driver =
{
    .description      = "usbfront_hc_driver",
    .product_desc     = "Xen USB Front-End Driver",
    .hcd_priv_size    = sizeof( struct usbfront_hcd ),
    .irq              = usbfront_hcd_irq,
    .flags            = HCD_USB2,
    /* reset optional. */
    .start            = usbfront_hcd_start,
    /* suspend optional. */
    /* resume optional. */
    .stop             = usbfront_hcd_stop,
    .get_frame_number = usbfront_hcd_get_frame_number,
    .urb_enqueue      = usbfront_hcd_urb_enqueue,
    .urb_dequeue      = usbfront_hcd_urb_dequeue,
    .endpoint_disable = usbfront_hcd_endpoint_disable,
    .hub_status_data  = usbfront_hcd_hub_status_data,
    .hub_control      = usbfront_hcd_hub_control,
    /* hub_suspend optional. */
    /* hub_resume optional. */
    .start_port_reset = usbfront_hcd_start_port_reset,
};

static void usbfront_hcd_invalid_stimulus
  ( struct usbfront_hcd * uhcd, usbfront_hcd_stimulus stimulus );

static void usbfront_hcd_kick_urbs
  ( struct usbfront_hcd * uhcd, unsigned long * flags );

static void usbfront_hcd_handle_stimulus
(
    struct usbfront_hcd   * uhcd,
    usbfront_hcd_stimulus   stimulus,
    unsigned long         * flags
)
{
    trace3
    (
        "uhcd %p in state %d received stimulus %d",
        uhcd,
        uhcd->state,
        stimulus
    );

    switch( uhcd->state )
    {
      case usbfront_hcd_state_i:
        switch( stimulus )
        {
          case usbfront_hcd_stimulus_uq:
            usbfront_hcd_kick_urbs( uhcd, flags );
            break;
        }
        break;
      default:
        usbfront_hcd_invalid_stimulus( uhcd, stimulus );
        break;
    }
}

static void usbfront_hcd_invalid_stimulus
  ( struct usbfront_hcd * uhcd, usbfront_hcd_stimulus stimulus )
{
    trace();

    printk
    (
        KERN_ERR "usbfront: hc_driver_uhcd %p in state %d"
        "received invalid stimulus %d",
        uhcd,
        uhcd->state,
        stimulus
    );
}

static void usbfront_hcd_kick_urbs
  ( struct usbfront_hcd * uhcd, unsigned long * flags )
{
    trace();

    struct usbfront_device * device =
      usbfront_hcd_hcd_to_usbfront_device( usbfront_hcd_to_hcd( uhcd ) );

    while
    (
        !usbfront_sll_is_empty( &uhcd->urb_sll )
        &&
        !list_empty( &uhcd->free_resource_list )
    )
    {
        trace0( "Starting URB" );

        struct urb * urb = container_of
        (
            (void **)usbfront_sll_remove_first( &uhcd->urb_sll ),
            struct urb,
            hcpriv
        );

        struct usbfront_hcd_resource * resource = list_entry
        (
            uhcd->free_resource_list.next,
            struct usbfront_hcd_resource,
            link
        );

        list_del_init( usbfront_hcd_resource_to_link( resource ) );

        /* FIXME: Hmm, calling this with the spinlock held. Not a problem    */
        /* unless we call back with a spinlock held.                         */

        usbfront_hcd_resource_start_urb( resource, urb, device );
    }
}

void usbfront_hcd_resource_completed
  ( struct usbfront_hcd * uhcd, struct usbfront_hcd_resource * resource )
{
    trace();

    unsigned long flags;

    spin_lock_irqsave( &uhcd->lock, flags );

    list_add( &resource->link, &uhcd->free_resource_list );

    usbfront_hcd_kick_urbs( uhcd, &flags );

    spin_unlock_irqrestore( &uhcd->lock, flags );
}

static irqreturn_t usbfront_hcd_irq
  ( struct usb_hcd * hcd, struct pt_regs * regs )
{
    trace();

    struct usbfront_hcd * uhcd = usbfront_hcd_hcd_to_uhcd( hcd );

    struct usbfront_device * device =
      usbfront_hcd_hcd_to_usbfront_device( hcd );

    struct usbfront_device_request * request;

    while
    (
        ( request = usbfront_device_collect_completed_request( device ) )
        !=
        NULL
    )
    {
        usbfront_hcd_resource_request_completed( request, uhcd, hcd, regs );
    }

    return IRQ_HANDLED;
}

static int usbfront_driver_probe_or_remove( struct device * dev, int remove )
{
    trace();

    int return_value = 0;

    struct usb_hcd * hcd;

    if( remove )
    {
        goto REMOVE;
    }

    if( usb_disabled() )
    {
        return_value = -ENODEV;

        goto REMOVE_0;
    }

    if( ( hcd = usb_create_hcd( &usbfront_hc_driver ) ) == NULL )
    {
        return_value = -ENOMEM;

        goto REMOVE_0;
    }

    dev_set_drvdata( dev, hcd );

    hcd->self.bus_name = dev->bus_id; /* FIXME: breaks encapsulation */
    hcd->self.controller = dev;       /* FIXME: breaks encapsulation */

    usbfront_device_install_irq_handler
      ( usbfront_device_dev_to( dev ), usb_hcd_irq, hcd );

    if( ( return_value = usb_register_bus( &hcd->self ) ) != 0 )
    {
        usb_put_hcd( hcd ); /* FIXME: not sure about this. */

        goto REMOVE_1;
    }

    if( ( return_value = hcd->driver->start( hcd ) ) != 0 )
    {
        goto REMOVE_2;
    }

    return 0;

  REMOVE:

    hcd = dev_get_drvdata( dev );

    if( HCD_IS_RUNNING( hcd->state ) )     /* FIXME: breaks encapsulation */
    {                                      /* FIXME: breaks encapsulation */
        hcd->state = USB_STATE_QUIESCING;  /* FIXME: breaks encapsulation */
    }                                      /* FIXME: breaks encapsulation */

    usb_disconnect( &hcd->self.root_hub );  /* FIXME: breaks encapsulation */

    hcd->driver->stop( hcd );

    hcd->state = USB_STATE_HALT; /* FIXME: breaks encapsulation */

  REMOVE_2:

    usb_deregister_bus( &hcd->self );

  REMOVE_1:

    usbfront_device_remove_irq_handler( usbfront_device_dev_to( dev ) );

    dev_set_drvdata( dev, NULL );

  REMOVE_0:

    return return_value;
}

static int usbfront_driver_probe( struct device * dev )
{
    trace();

    return usbfront_driver_probe_or_remove( dev, 0 );
}

static int usbfront_driver_remove( struct device * dev )
{
    trace();

    return usbfront_driver_probe_or_remove( dev, 1 );
}

static void usbfront_driver_shutdown( struct device * dev )
{
    trace();
}

static struct device_driver usbfront_driver =
{
    .name     = "usbfront_driver",
    .bus      = &usbfront_bus_type,
    .owner    = THIS_MODULE,
    .probe    = usbfront_driver_probe,
    .remove   = usbfront_driver_remove,
    .shutdown = usbfront_driver_shutdown,
};

int usbfront_driver_init( void )
{
    trace();

    return driver_register( &usbfront_driver );
}

void usbfront_driver_exit( void )
{
    trace();

    driver_unregister( &usbfront_driver );
}
