/*****************************************************************************/
/* A resource used by the usbfront host controller device driver to process  */
/* an URB by translating the URB and optional unlink request into            */
/* usbfront_device_requests for submission to the back end.                  */
/*                                                                           */
/* Copyright (c) 2005 Harry Butterworth IBM Corporation                      */
/*                                                                           */
/* This program is free software; you can redistribute it and/or modify it   */
/* under the terms of the GNU General Public License as published by the     */
/* Free Software Foundation; either version 2 of the License, or (at your    */
/* option) any later version.                                                */
/*                                                                           */
/* This program is distributed in the hope that it will be useful, but       */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General */
/* Public License for more details.                                          */
/*                                                                           */
/* You should have received a copy of the GNU General Public License along   */
/* with this program; if not, write to the Free Software Foundation, Inc.,   */
/* 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA                   */
/*                                                                           */
/*****************************************************************************/

#ifndef USBFRONT_HCD_RESOURCE_H
#define USBFRONT_HCD_RESOURCE_H

#include <asm/atomic.h>
#include <linux/usb.h>
#include <linux/list.h>
#include "usbfront_device.h"

struct usbfront_hcd;
struct usbfront_hcd_resource;

struct usbfront_hcd_resource_request
{
    struct usbfront_hcd_resource   * resource;
    struct usbfront_device_request   request;
};

static inline struct usbfront_hcd_resource_request *
  usbfront_hcd_resource_request_device_request_to_resource_request
    ( struct usbfront_device_request * request )
{
    return container_of
      ( request, struct usbfront_hcd_resource_request, request );
}

struct usbfront_hcd_resource
{
    struct list_head                       link;
    int                                    index;
    usbif_iso_t                          * schedule;
    struct urb                           * urb;
    atomic_t                               requests_out;
    struct usbfront_hcd_resource_request   enqueue_request;
    struct usbfront_hcd_resource_request   dequeue_request;
};

static inline struct list_head * usbfront_hcd_resource_to_link
  ( struct usbfront_hcd_resource * resource )
{
    return &resource->link;
}

int usbfront_hcd_resource_init
  ( struct usbfront_hcd_resource * resource, int index );

void usbfront_hcd_resource_exit
  ( struct usbfront_hcd_resource * resource );

void usbfront_hcd_resource_start_urb
(
    struct usbfront_hcd_resource * resource,
    struct urb                   * urb,
    struct usbfront_device       * device
);

void usbfront_hcd_resource_request_completed
(
    struct usbfront_device_request * request,
    struct usbfront_hcd            * uhcd,
    struct usb_hcd                 * hcd,
    struct pt_regs                 * regs
);

void usbfront_hcd_resource_dequeue_urb
  ( struct urb * urb, struct usbfront_device * device );

#endif
