/*****************************************************************************/
/* Front-end module for Xen USB split driver.                                */
/*                                                                           */
/* Copyright (c) 2005 Harry Butterworth IBM Corporation                      */
/*                                                                           */
/* This program is free software; you can redistribute it and/or modify it   */
/* under the terms of the GNU General Public License as published by the     */
/* Free Software Foundation; either version 2 of the License, or (at your    */
/* option) any later version.                                                */
/*                                                                           */
/* This program is distributed in the hope that it will be useful, but       */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General */
/* Public License for more details.                                          */
/*                                                                           */
/* You should have received a copy of the GNU General Public License along   */
/* with this program; if not, write to the Free Software Foundation, Inc.,   */
/* 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA                   */
/*                                                                           */
/*****************************************************************************/

#include <linux/module.h>
#include <linux/usb.h>
#include "usbfront_bus.h"
#include "usbfront_device.h"
#include "usbfront_discovery.h"
#include "usbfront_driver.h"
#include "usbfront_trace.h"

static int usbfront_module_init_or_exit( int exit )
{
    trace();

    int return_value = 0;

    if( usb_disabled() )
    {
        return_value = -ENODEV;

        goto EXIT_NO_USB;
    }

    if( exit )
    {
        goto EXIT;
    }

    if( ( return_value = usbfront_bus_init() ) != 0 )
    {
        goto EXIT_NO_BUS;
    }

    if( ( return_value = usbfront_driver_init() ) != 0 )
    {
        goto EXIT_NO_DRIVER;
    }

    if( ( return_value = usbfront_discovery_init() ) != 0 )
    {
        goto EXIT_NO_DISCOVERY;
    }

    return 0;

  EXIT:

    usbfront_discovery_exit();

  EXIT_NO_DISCOVERY:

    usbfront_driver_exit();

  EXIT_NO_DRIVER:

    usbfront_bus_exit();

  EXIT_NO_BUS:
  EXIT_NO_USB:

    return return_value;
}

static int __init usbfront_module_init( void )
{
    trace();

    return usbfront_module_init_or_exit( 0 );
}

static void __exit usbfront_module_exit( void )
{
    trace();

    (void)usbfront_module_init_or_exit( 1 );
}

module_init( usbfront_module_init );
module_exit( usbfront_module_exit );

MODULE_LICENSE( "GPL" );
