/*****************************************************************************/
/* A singly linked list implementation                                       */
/* Copyright (c) 2005 Harry Butterworth IBM Corporation                      */
/*                                                                           */
/* This program is free software; you can redistribute it and/or modify it   */
/* under the terms of the GNU General Public License as published by the     */
/* Free Software Foundation; either version 2 of the License, or (at your    */
/* option) any later version.                                                */
/*                                                                           */
/* This program is distributed in the hope that it will be useful, but       */
/* WITHOUT ANY WARRANTY; without even the implied warranty of                */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General */
/* Public License for more details.                                          */
/*                                                                           */
/* You should have received a copy of the GNU General Public License along   */
/* with this program; if not, write to the Free Software Foundation, Inc.,   */
/* 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA                   */
/*                                                                           */
/*****************************************************************************/

#ifndef USBFRONT_SLL_H
#define USBFRONT_SLL_H

#include "usbfront_assert.h"

struct usbfront_slk
{
    struct usbfront_slk * next;
};

struct usbfront_sll
{
    struct usbfront_slk * first;
    struct usbfront_slk * last;
};

static inline void usbfront_slk_init( struct usbfront_slk * slk )
{
    slk->next = slk;
}

static inline int usbfront_slk_is_singular( struct usbfront_slk * slk )
{
    return ( slk->next == slk );
}

static inline void usbfront_sll_init( struct usbfront_sll * sll )
{
    sll->first = 0;
}

static inline int usbfront_sll_is_empty( struct usbfront_sll * sll )
{
    return ( sll->first == 0 );
}

static inline void usbfront_sll_add_last
  ( struct usbfront_sll * sll, struct usbfront_slk * slk )
{
    ASSERT( usbfront_slk_is_singular( slk ) );

    slk->next = 0;

    if( sll->first != 0 )
    {
        sll->last->next = slk;
    }
    else
    {
        sll->first = slk;
    }

    sll->last = slk;
}

static inline struct usbfront_slk * usbfront_sll_remove_first
  ( struct usbfront_sll * sll )
{
    struct usbfront_slk * slk = sll->first;

    if( slk != 0 )
    {
        sll->first = slk->next;

        slk->next = slk;
    }

    return slk;
}

/* Returns 1 if slk removed, 0 otherwise. */

static inline int usbfront_sll_remove_slk
  ( struct usbfront_sll * sll, struct usbfront_slk * slk )
{
    struct usbfront_slk * prev_slk = NULL;
    struct usbfront_slk * next_slk = sll->first;

    while( next_slk != NULL )
    {
        if( next_slk != slk )
        {
            prev_slk = next_slk;
            next_slk = next_slk->next;
        }
        else
        {
            next_slk = next_slk->next;

            if( prev_slk != NULL )
            {
                prev_slk->next = next_slk;
            }
            else
            {
                sll->first = next_slk;
            }

            if( next_slk == NULL )
            {
                sll->last = prev_slk;
            }

            slk->next = slk;

            return 1;
        }
    }

    return 0;
}

#endif
